#version 120

const int gcolorFormat = 1; // Internal - RGB8
const int gdepthFormat = 1; // Internal - RGB8
/* GNORMALFORMAT:RGBA32F */
//const int gnormalFormat = 2; // Internal - RGB16
const int compositeFormat = 1; // Internal - RGB8
const float	ambientOcclusionLevel = 0.019f;   // Internal
const float	sunPathRotation	= -18.0f;   // Internal

const int shadowMapResolution = 2048;   // Internal
const float shadowDistance = 128.0f; // Internal

#define SUNLIGHTAMOUNT 1.2f

const vec3 torchcolor = vec3(1.0f,0.7f,0.5f);

#define TORCH_POWER 3.5f
#define TORCH_INTENSITY 2.0f

varying vec4 texcoord;
varying vec4 lmcoord;
varying vec3 lightVector;
varying vec3 lightOffset0;
varying vec3 lightOffset1;
varying vec3 lightOffset2;
varying vec3 lightOffset3;
varying vec3 lightOffset4;
varying vec3 lightOffset5;
varying vec3 sunlight_color;
varying vec3 ambient_color;
varying vec3 heldLightSpecMultiplier;
varying float heldLightMagnitude;
varying float transition_fading;
varying float ambient_transition;

uniform sampler2D gcolor;
uniform sampler2D gdepth;   // specularity - red green blue gloss
uniform sampler2D gnormal;
uniform sampler2D composite;
uniform sampler2D gaux1;
uniform sampler2D depthtex0;
uniform sampler2D shadow;
uniform sampler2D watershadow;
uniform sampler2D shadowcolor;

uniform mat4 gbufferProjection;
uniform mat4 gbufferModelViewInverse;
uniform mat4 shadowProjection;
uniform mat4 shadowModelView;

uniform float near;
uniform float rainStrength;

uniform int isEyeInWater;

float gp22 = gbufferProjection[2][2] * 0.5f;
float gp32 = gbufferProjection[3][2] * 0.5f;
float igp00 = 2.0f / gbufferProjection[0][0];
float igp11 = 2.0f / gbufferProjection[1][1];

vec3 invproj(in vec3 p) {
    vec3 pos = p - 0.5f;
    float z = gp32 / (pos.z + gp22);
    return vec3(pos.x * igp00, pos.y * igp11, -1.0f) * z;
}

float nspec(in vec3 pos, in vec3 lvector, in vec3 normal) {
    
    // half vector
    vec3 cHalf = normalize(lvector + pos);
    float normalDotHalf = dot(normal, cHalf);
    
    // beckmann's distribution function D
    float normalDotHalf2 = normalDotHalf * normalDotHalf;
    float roughness2 = 0.25f;
    float exponent = -(1.0f - normalDotHalf2) / (normalDotHalf2 * roughness2);
    const float e = 2.71828182846f;
    float D = pow(e, exponent) / (roughness2 * normalDotHalf2 * normalDotHalf2);
	
    // fresnel term F
    float normalDotEye = dot(normal, pos);
    float F = pow(1.0f + normalDotEye, 3.0f);
    
    // self shadowing term G
    float normalDotLight = dot(normal, lvector);
    float X = 2.0f * normalDotHalf / dot(pos, cHalf);
    float G = min(1.0f, min(X * normalDotLight, X * normalDotEye));
    const float pi = 3.1415927f;
    float CookTorrance = (D * F * G) / (pi * normalDotEye);
	
    return max(CookTorrance / pi, 0.00001f);
}

float xspec(in vec3 pos, in vec3 lvector, in vec3 normal, in vec3 lvector_alt0, in vec3 lvector_alt1) {

    // half vector
    vec3 cHalf = normalize(lvector + pos);
    float normalDotHalf = dot(normal, cHalf);
    vec3 lvec = lvector;
    
    // expand lighting area
    vec3 cHalf_alt = normalize(lvector_alt0 + pos);
    float normalDotHalf_alt = dot(normal, cHalf_alt);
    if (normalDotHalf_alt > normalDotHalf) {
        lvec = lvector_alt0;
        normalDotHalf = normalDotHalf_alt;
    }
    cHalf_alt = normalize(lvector_alt1 + pos);
    normalDotHalf_alt = dot(normal, cHalf_alt);
    if (normalDotHalf_alt > normalDotHalf) {
        lvec = lvector_alt1;
    }
    return nspec(pos, lvec, normal);
}

//////////////////////////////main//////////////////////////////

void main() {
    vec3 color = texture2D(gcolor, texcoord.st).rgb;
    vec3 compo = texture2D(composite, texcoord.st).rgb;
    float spec = 0.0f;
    
    if (isEyeInWater < 1) {
        vec3 aux = texture2D(gaux1, texcoord.st).rgb;
        const float fland = 0.05f;
        if (aux.g > fland) {
            vec3 total_effect = mix(vec3(0.22f), ambient_color, 0.22f); // sky and indoor ambient light

            float torch_lightmap = pow(aux.b, TORCH_POWER) * TORCH_INTENSITY;   //torch lightmaps
            total_effect += torch_lightmap * torchcolor;    // torch light

            vec3 fragposition = invproj(vec3(texcoord.s, texcoord.t, texture2D(depthtex0, texcoord.st).r));
            float dist = length(fragposition);
            float hand = 1.0f;
            if(aux.g > 0.35f && aux.g < 0.45f) {
                hand = 0.2f;
            }
            vec3 npos = normalize(fragposition);
            vec3 normal = texture2D(gnormal, texcoord.st).rgb * 2.0f - 1.0f;
            vec3 nnormal = normalize(normal);
            total_effect += max(dot(nnormal, -npos), 0.0f) * hand * heldLightSpecMultiplier * heldLightMagnitude / dist;    // held light

            vec3 shading = vec3(1.0f);
            float translucent = 0.0f;
            if(aux.g > 0.25f && aux.g < 0.35f) {
                translucent = 1.0f;
            }
            if (dist < shadowDistance) {
                vec4 worldposition = gbufferModelViewInverse * vec4(fragposition, 1.0f);
                worldposition = shadowModelView * worldposition;
                float behindcamera = -worldposition.w;
                float comparedepth = -worldposition.z;
                vec4 projectedshadow = shadowProjection * worldposition;
                projectedshadow /= projectedshadow.w;
                vec2 shadowPos = projectedshadow.st * 0.5f + 0.5f;
                const float fshadow = 0.15f;
                if (aux.g > fshadow && behindcamera < 0.0f && shadowPos.s < 1.0f && shadowPos.s > 0.0f && shadowPos.t < 1.0f && shadowPos.t > 0.0f) {
                    //Shadow sampling
                    const vec2 circle_offset0 = vec2(-0.875f, 0.875f);
                    const vec2 circle_offset1 = vec2(-0.5f, 0.125f);
                    const vec2 circle_offset2 = vec2(-0.125f, -0.75f);
                    const vec2 circle_offset3 = vec2(0.375f, 0.5f);
                    const vec2 circle_offset4 = vec2(0.75f, -0.375f);
                    float offset = 1.0f / float(shadowMapResolution);
                    
                    float shadingwater = 0.0f;
                    shadingwater += clamp(comparedepth - (0.05f + texture2D(watershadow, shadowPos + circle_offset0 * offset).z * 255.95f), 0.0f, 1.0f);
                    shadingwater += clamp(comparedepth - (0.05f + texture2D(watershadow, shadowPos + circle_offset1 * offset).z * 255.95f), 0.0f, 1.0f);
                    shadingwater += clamp(comparedepth - (0.05f + texture2D(watershadow, shadowPos + circle_offset2 * offset).z * 255.95f), 0.0f, 1.0f);
                    shadingwater += clamp(comparedepth - (0.05f + texture2D(watershadow, shadowPos + circle_offset3 * offset).z * 255.95f), 0.0f, 1.0f);
                    shadingwater += clamp(comparedepth - (0.05f + texture2D(watershadow, shadowPos + circle_offset4 * offset).z * 255.95f), 0.0f, 1.0f);
                    shadingwater = 1.0f - shadingwater * 0.2f;
                    
                    float shadingsharp = 0.0f;
                    shadingsharp += clamp(comparedepth - (0.05f + (texture2D(shadow, shadowPos + circle_offset0 * offset).z) * 255.95f), 0.0f, 1.0f);
                    shadingsharp += clamp(comparedepth - (0.05f + (texture2D(shadow, shadowPos + circle_offset1 * offset).z) * 255.95f), 0.0f, 1.0f);
                    shadingsharp += clamp(comparedepth - (0.05f + (texture2D(shadow, shadowPos + circle_offset2 * offset).z) * 255.95f), 0.0f, 1.0f);
                    shadingsharp += clamp(comparedepth - (0.05f + (texture2D(shadow, shadowPos + circle_offset3 * offset).z) * 255.95f), 0.0f, 1.0f);
                    shadingsharp += clamp(comparedepth - (0.05f + (texture2D(shadow, shadowPos + circle_offset4 * offset).z) * 255.95f), 0.0f, 1.0f);
                    shadingsharp = 1.0f - shadingsharp * 0.2f;

                    vec3 shadowColorSample = texture2D(shadowcolor, shadowPos).rgb;
                    shading = shadowColorSample * (shadingsharp - shadingwater) + shadingwater;
                }
            }
            shading = clamp(shading, 0.0f, 1.0f);
            float NdotL = dot(normal, lightVector);
            float sss_transparency = mix(0.0f, 0.75f, translucent); //subsurface scattering amount
            float sss = pow(max(dot(npos, lightVector), 0.0f), 20.0f) * sss_transparency * clamp(-NdotL, 0.0f, 1.0f) * translucent * 4.0f;
            float distof2 = clamp(1.0f - dist / (shadowDistance * 0.75f), 0.0f, 1.0f);
            float sss_fade = pow(distof2, 0.2f);
            sss = mix(0.0f, sss, min(sss_fade + 0.5f, 1.0f));
            total_effect += sss * sunlight_color * shading * (1.0f - rainStrength * 0.9f) * ambient_transition; // sub surface scattering light
            
            float distof = clamp(1.0f - dist / shadowDistance, 0.0f, 1.0f);
            float shadow_fade = clamp(distof * 12.0f, 0.0f, 1.0f);
            float sunlight_direct = max(NdotL, 0.0f);
            sunlight_direct = mix(sunlight_direct, 0.75f, translucent);
            total_effect += sunlight_color * (1.0f - rainStrength * 0.9f) * mix(vec3(1.0f), shading, shadow_fade) * SUNLIGHTAMOUNT * sunlight_direct * transition_fading;    // direct sun light
            
            // spectral reflection on water - for sun and moon only
            const float fspec = 0.85f;
            if (aux.g > fspec) {
                if (transition_fading > 0.999f) {
                    spec = nspec(-npos, lightVector, nnormal);
                }
                else {
                    spec = xspec(-npos, lightOffset0, nnormal, lightOffset1, lightOffset2) + xspec(-npos, lightOffset3, nnormal, lightOffset4, lightOffset5);
                }
                spec *= (1.0f - rainStrength);
            }
            else {
                if (dot(nnormal, lightVector) > 0.0f) {
                    vec4 specularity = texture2D(gdepth, texcoord.st);
                    total_effect += pow(nspec(-npos, lightVector, nnormal), specularity.a) * specularity.rgb * shading * (hand - 0.2f);
                }
            }
            
            color *= total_effect;
            compo *= total_effect;
        }
        else {
            // affect block breaking color when raining
            color = mix(color, gl_Fog.color.rgb, rainStrength * 0.1f);
            compo = mix(compo, gl_Fog.color.rgb, rainStrength * 0.1f);
        }
    }
    color = clamp(color, 0.0f, 1.0f);
    compo = clamp(compo, 0.0f, 1.0f);

/* DRAWBUFFERS:03 */
    gl_FragData[0] = vec4(color, spec);	// gcolor in composite1.fsh
    gl_FragData[1] = vec4(compo, spec);	// composite in composite1.fsh
}

//////////////////////////////main//////////////////////////////
