#version 120

#define DOF
#define USE_HIGH_QUALITY_BLUR

#define SUNRAYS
#define MOONRAYS

#define BLOOM
#define BLOOM_AMOUNT 3.0f

#define LENS

#define CROSSPROCESS
#define CROSSPROCESS_AMOUNT 0.5f

varying vec4 texcoord;

varying vec2 lightPos;
varying vec2 moonPos;

uniform sampler2D gcolor;
uniform sampler2D depthtex0;

uniform vec3 moonPosition;  // used by moon rays
uniform vec3 sunPosition;   // used by lens and sun rays

uniform float aspectRatio;
uniform float near;
uniform float far;
uniform float viewWidth;
uniform float viewHeight;
uniform float rainStrength;

uniform int worldTime;

float pw = 1.0f / viewWidth;
float ph = 1.0f / viewHeight;

float getDepth(in vec2 coord) {
    return 2.0f * near * far / (far + near - (2.0f * texture2D(depthtex0, coord).x - 1.0f) * (far - near));
}

#ifdef DOF
// HYPERFOCAL = (Focal Distance ^ 2)/(Circle of Confusion * F Stop) + Focal Distance
const float HYPERFOCAL = 3.132f;
const float PICONSTANT = 3.14159f;
vec4 getBlurredColor();
vec4 getSampleWithBoundsCheck(vec2 offset);
float samples = 0.0f;
#endif

const float GR_DECAY = 0.9f;
const float GR_DENSITY = 0.8f;
const float GR_EXPOSURE = 0.3f;
const float GR_FREQUENCY = 0.05f;

//////////////////////////////main//////////////////////////////

void main() {
	vec4 color = texture2D(gcolor, texcoord.st);
#ifdef DOF
	float depth = getDepth(texcoord.st);
	float cursorDepth = getDepth(vec2(0.5f, 0.5f));
    
    // foreground blur = 1/2 background blur. Blur should follow exponential pattern until cursor = hyperfocal -- Cursor before hyperfocal
    // Blur should go from 0 to 1/2 hyperfocal then clear to infinity -- Cursor @ hyperfocal.
    // hyperfocal to inifity is clear though dof extends from 1/2 hyper to hyper -- Cursor beyond hyperfocal
    float mixAmount = 0.0f;
    if (depth < cursorDepth) {
    	mixAmount = clamp(2.0f * ((min(cursorDepth, HYPERFOCAL) - depth) / min(cursorDepth, HYPERFOCAL)), 0.0f, 0.66f);
	} else {
		mixAmount =  1.0f - clamp(2.0f - depth / cursorDepth + depth / HYPERFOCAL - cursorDepth / HYPERFOCAL, 0.0f, 0.66f);
	}
    color = mix(color, getBlurredColor(), mixAmount);
#endif
#ifdef BLOOM
	int j;
	int i;
	vec4 sum = vec4(0.0f);
    	float count = 0.0f;
    	for( i= -4 ;i < 4; i++) {
        	for (j = -3; j < 3; j++) {
            		vec2 coord = texcoord.st + vec2(j,i) * 0.004f;
            		if(coord.x > 0.0f && coord.x < 1.0f && coord.y > 0.0f && coord.y < 1.0f){
                		sum += texture2D(gcolor, coord) * BLOOM_AMOUNT;
                		count += 1.0f;
            		}
        	}
    	}
    	sum = sum / vec4(count);
	if (color.r < 0.3f) {
		color += sum * sum * 0.012f;
	}
	else {
		if (color.r < 0.5f) {
			color += sum * sum * 0.009f;
		}
		else {
			color += sum * sum * 0.0075f;
		}
	}
#endif
#ifdef SUNRAYS
    if ((worldTime > 22350 || worldTime < 14200) && sunPosition.z < 0.0f) {
        vec2 tex = texcoord.st;
        vec2 delta = (lightPos - tex);
        float decay = pow(max(1.0f - length(delta), 0.0f), GR_DECAY);
        delta *= GR_DENSITY * GR_FREQUENCY;
        vec3 rays = vec3(0.0f);
        tex += delta;
        rays += texture2D(depthtex0, tex).x * texture2D(gcolor, tex).rgb;
        tex += delta;
        rays += texture2D(depthtex0, tex).x * texture2D(gcolor, tex).rgb;
        tex += delta;
        rays += texture2D(depthtex0, tex).x * texture2D(gcolor, tex).rgb;
        tex += delta;
        rays += texture2D(depthtex0, tex).x * texture2D(gcolor, tex).rgb;
        tex += delta;
        rays += texture2D(depthtex0, tex).x * texture2D(gcolor, tex).rgb;
        tex += delta;
        rays += texture2D(depthtex0, tex).x * texture2D(gcolor, tex).rgb;
        tex += delta;
        rays += texture2D(depthtex0, tex).x * texture2D(gcolor, tex).rgb;
        tex += delta;
        rays += texture2D(depthtex0, tex).x * texture2D(gcolor, tex).rgb;
        tex += delta;
        rays += texture2D(depthtex0, tex).x * texture2D(gcolor, tex).rgb;
        tex += delta;
        rays += texture2D(depthtex0, tex).x * texture2D(gcolor, tex).rgb;
        
        tex += delta;
        rays += texture2D(depthtex0, tex).x * texture2D(gcolor, tex).rgb;
        tex += delta;
        rays += texture2D(depthtex0, tex).x * texture2D(gcolor, tex).rgb;
        tex += delta;
        rays += texture2D(depthtex0, tex).x * texture2D(gcolor, tex).rgb;
        tex += delta;
        rays += texture2D(depthtex0, tex).x * texture2D(gcolor, tex).rgb;
        tex += delta;
        rays += texture2D(depthtex0, tex).x * texture2D(gcolor, tex).rgb;
        tex += delta;
        rays += texture2D(depthtex0, tex).x * texture2D(gcolor, tex).rgb;
        tex += delta;
        rays += texture2D(depthtex0, tex).x * texture2D(gcolor, tex).rgb;
        tex += delta;
        rays += texture2D(depthtex0, tex).x * texture2D(gcolor, tex).rgb;
        tex += delta;
        rays += texture2D(depthtex0, tex).x * texture2D(gcolor, tex).rgb;
        tex += delta;
        rays += texture2D(depthtex0, tex).x * texture2D(gcolor, tex).rgb;
        
        color.rgb += rays * GR_FREQUENCY * GR_EXPOSURE * (1.0f - rainStrength * 0.95f) * (clamp((14200.0f - worldTime) / 600.0f, 0.0f, 1.0f) + clamp((worldTime - 22350.0f) / 400.0f, 0.0f, 1.0f)) * decay;
    }
#endif
#ifdef MOONRAYS
    if ((worldTime > 12050 || worldTime < 600) && moonPosition.z < 0.0f) {
        vec2 tex = texcoord.st;
        vec2 delta = (moonPos - tex);
        float decay = pow(max(1.0f - length(delta), 0.0f), GR_DECAY);
        delta *= GR_DENSITY * GR_FREQUENCY;
        vec3 rays = vec3(0.0f);
        tex += delta;
        rays += texture2D(depthtex0, tex).x * texture2D(gcolor, tex).rgb;
        tex += delta;
        rays += texture2D(depthtex0, tex).x * texture2D(gcolor, tex).rgb;
        tex += delta;
        rays += texture2D(depthtex0, tex).x * texture2D(gcolor, tex).rgb;
        tex += delta;
        rays += texture2D(depthtex0, tex).x * texture2D(gcolor, tex).rgb;
        tex += delta;
        rays += texture2D(depthtex0, tex).x * texture2D(gcolor, tex).rgb;
        tex += delta;
        rays += texture2D(depthtex0, tex).x * texture2D(gcolor, tex).rgb;
        tex += delta;
        rays += texture2D(depthtex0, tex).x * texture2D(gcolor, tex).rgb;
        tex += delta;
        rays += texture2D(depthtex0, tex).x * texture2D(gcolor, tex).rgb;
        tex += delta;
        rays += texture2D(depthtex0, tex).x * texture2D(gcolor, tex).rgb;
        tex += delta;
        rays += texture2D(depthtex0, tex).x * texture2D(gcolor, tex).rgb;
        
        tex += delta;
        rays += texture2D(depthtex0, tex).x * texture2D(gcolor, tex).rgb;
        tex += delta;
        rays += texture2D(depthtex0, tex).x * texture2D(gcolor, tex).rgb;
        tex += delta;
        rays += texture2D(depthtex0, tex).x * texture2D(gcolor, tex).rgb;
        tex += delta;
        rays += texture2D(depthtex0, tex).x * texture2D(gcolor, tex).rgb;
        tex += delta;
        rays += texture2D(depthtex0, tex).x * texture2D(gcolor, tex).rgb;
        tex += delta;
        rays += texture2D(depthtex0, tex).x * texture2D(gcolor, tex).rgb;
        tex += delta;
        rays += texture2D(depthtex0, tex).x * texture2D(gcolor, tex).rgb;
        tex += delta;
        rays += texture2D(depthtex0, tex).x * texture2D(gcolor, tex).rgb;
        tex += delta;
        rays += texture2D(depthtex0, tex).x * texture2D(gcolor, tex).rgb;
        tex += delta;
        rays += texture2D(depthtex0, tex).x * texture2D(gcolor, tex).rgb;
        
        color.rgb += rays * 0.5f * GR_FREQUENCY * GR_EXPOSURE * (1.0f - rainStrength * 0.95f) * (clamp((600.0f - worldTime) / 600.0f, 0.0f, 1.0f) + clamp((worldTime - 12050.0f) / 400.0f, 0.0f, 1.0f)) * decay;
    }
#endif
#ifdef LENS
    const float flaremultR = 0.8f;
    const float flaremultG = 1.0f;
    const float flaremultB = 1.5f;

    float sunmask = texture2D(gcolor, vec2(0.0f)).a;
    if (sunmask > 0.0f) {
        // Red glow dot
        vec2 flare11scale = vec2(2.0f, 2.0f);
        float flare11offset = -0.523f;
        vec2 flare11pos = vec2(((1.0f - lightPos.x) * (flare11offset + 1.0f) - (flare11offset * 0.5f)) * aspectRatio * flare11scale.x, ((1.0f - lightPos.y) * (flare11offset + 1.0f) - (flare11offset * 0.5f)) * flare11scale.y);
        float flare11 = distance(flare11pos, vec2(texcoord.s * aspectRatio * flare11scale.x, texcoord.t * flare11scale.y));
        flare11 = 0.5f - flare11;
        flare11 = clamp(flare11 * 2.0f, 0.0f, 1.0f) * clamp(-sunPosition.z, 0.0f, 1.0f);
        flare11 = pow(flare11, 3.5f);
        flare11 *= sunmask;
        color.r += flare11 * 1.0f * flaremultR;
        color.g += flare11 * 0.2f * flaremultG;
        
        // Purple glow dot
        vec2 flare12scale = vec2(1.5f, 1.5f);
        float flare12offset = -0.123f;
        vec2 flare12pos = vec2(((1.0f - lightPos.x) * (flare12offset + 1.0f) - (flare12offset * 0.5f)) * aspectRatio * flare12scale.x, ((1.0f - lightPos.y) * (flare12offset + 1.0f) - (flare12offset * 0.5f)) * flare12scale.y);
        float flare12 = distance(flare12pos, vec2(texcoord.s * aspectRatio * flare12scale.x, texcoord.t * flare12scale.y));
        flare12 = 0.5f - flare12;
        flare12 = clamp(flare12 * 2.0f, 0.0f, 1.0f) * clamp(-sunPosition.z, 0.0f, 1.0f);
        flare12 = pow(flare12, 3.5f);
        flare12 *= sunmask * 0.7f;
        color.r += flare12 * 0.8f * flaremultR;
        color.g += flare12 * 0.2f * flaremultG;
        color.b += flare12 * 1.0f * flaremultB;
    }
#endif
#ifdef CROSSPROCESS
    color.r += CROSSPROCESS_AMOUNT * (color.r * 0.15f - 0.02f);
    color.g += CROSSPROCESS_AMOUNT * (color.g * 0.20f - 0.02f);
    color.b += CROSSPROCESS_AMOUNT * (color.b * -0.2f + 0.05f);
#endif
    gl_FragColor = vec4(color.rgb, 1.0f);
}

//////////////////////////////main//////////////////////////////

#ifdef DOF
vec4 getBlurredColor() {
	vec4 blurredColor = vec4(0.0f);
	float depth = getDepth(texcoord.xy);
	vec2 aspectCorrection = vec2(1.0f, aspectRatio) * 0.005f;
	vec2 ac0_4 = 0.4f * aspectCorrection;	// 0.4f
#ifdef USE_HIGH_QUALITY_BLUR
	vec2 ac0_4x0_4 = 0.4f * ac0_4;		// 0.16f
	vec2 ac0_4x0_7 = 0.7f * ac0_4;		// 0.28f
#endif
	vec2 ac0_29 = 0.29f * aspectCorrection;	// 0.29f
#ifdef USE_HIGH_QUALITY_BLUR
	vec2 ac0_29x0_7 = 0.7f * ac0_29;	// 0.203f
	vec2 ac0_29x0_4 = 0.4f * ac0_29;	// 0.116f
#endif
	vec2 ac0_15 = 0.15f * aspectCorrection;	// 0.15f
	vec2 ac0_37 = 0.37f * aspectCorrection;	// 0.37f
#ifdef USE_HIGH_QUALITY_BLUR
	vec2 ac0_15x0_9 = 0.9f * ac0_15;	// 0.135f
	vec2 ac0_37x0_9 = 0.37f * ac0_37;	// 0.1369f
#endif
	vec2 lowSpace = texcoord.st;
	vec2 highSpace = 1.0f - lowSpace;
	vec2 space = vec2(min(lowSpace.s, highSpace.s), min(lowSpace.t, highSpace.t));
	if (space.s >= ac0_4.s && space.t >= ac0_4.t) {
		blurredColor += texture2D(gcolor, texcoord.st + vec2(0.0f, ac0_4.t));
		blurredColor += texture2D(gcolor, texcoord.st + vec2(ac0_4.s, 0.0f));
		blurredColor += texture2D(gcolor, texcoord.st + vec2(0.0f, -ac0_4.t));
		blurredColor += texture2D(gcolor, texcoord.st + vec2(-ac0_4.s, 0.0f));
#ifdef USE_HIGH_QUALITY_BLUR
		blurredColor += texture2D(gcolor, texcoord.st + vec2(ac0_4x0_7.s, 0.0f));
		blurredColor += texture2D(gcolor, texcoord.st + vec2(0.0f, -ac0_4x0_7.t));
		blurredColor += texture2D(gcolor, texcoord.st + vec2(-ac0_4x0_7.s, 0.0f));
		blurredColor += texture2D(gcolor, texcoord.st + vec2(0.0f, ac0_4x0_7.t));
		blurredColor += texture2D(gcolor, texcoord.st + vec2(ac0_4x0_4.s, 0.0f));
		blurredColor += texture2D(gcolor, texcoord.st + vec2(0.0f, -ac0_4x0_4.t));
		blurredColor += texture2D(gcolor, texcoord.st + vec2(-ac0_4x0_4.s, 0.0f));
		blurredColor += texture2D(gcolor, texcoord.st + vec2(0.0f, ac0_4x0_4.t));
#endif
		blurredColor += texture2D(gcolor, texcoord.st + vec2(ac0_29.s, -ac0_29.t));
		blurredColor += texture2D(gcolor, texcoord.st + vec2(ac0_29.s, ac0_29.t));
		blurredColor += texture2D(gcolor, texcoord.st + vec2(-ac0_29.s, ac0_29.t));
		blurredColor += texture2D(gcolor, texcoord.st + vec2(-ac0_29.s, -ac0_29.t));
#ifdef USE_HIGH_QUALITY_BLUR
		blurredColor += texture2D(gcolor, texcoord.st + vec2(ac0_29x0_7.s, ac0_29x0_7.t));
		blurredColor += texture2D(gcolor, texcoord.st + vec2(ac0_29x0_7.s, -ac0_29x0_7.t));
		blurredColor += texture2D(gcolor, texcoord.st + vec2(-ac0_29x0_7.s, ac0_29x0_7.t));
		blurredColor += texture2D(gcolor, texcoord.st + vec2(-ac0_29x0_7.s, -ac0_29x0_7.t));
		blurredColor += texture2D(gcolor, texcoord.st + vec2(ac0_29x0_4.s, ac0_29x0_4.t));
		blurredColor += texture2D(gcolor, texcoord.st + vec2(ac0_29x0_4.s, -ac0_29x0_4.t));
		blurredColor += texture2D(gcolor, texcoord.st + vec2(-ac0_29x0_4.s, ac0_29x0_4.t));
		blurredColor += texture2D(gcolor, texcoord.st + vec2(-ac0_29x0_4.s, -ac0_29x0_4.t));
#endif
		blurredColor += texture2D(gcolor, texcoord.st + vec2(ac0_15.s, ac0_37.t));
		blurredColor += texture2D(gcolor, texcoord.st + vec2(-ac0_37.s, ac0_15.t));
		blurredColor += texture2D(gcolor, texcoord.st + vec2(ac0_37.s, -ac0_15.t));
		blurredColor += texture2D(gcolor, texcoord.st + vec2(-ac0_15.s, -ac0_37.t));
		blurredColor += texture2D(gcolor, texcoord.st + vec2(-ac0_15.s, ac0_37.t));
		blurredColor += texture2D(gcolor, texcoord.st + vec2(ac0_37.s, ac0_15.t));
		blurredColor += texture2D(gcolor, texcoord.st + vec2(-ac0_37.s, -ac0_15.t));
		blurredColor += texture2D(gcolor, texcoord.st + vec2(ac0_15.s, -ac0_37.t));
#ifdef USE_HIGH_QUALITY_BLUR
		blurredColor += texture2D(gcolor, texcoord.st + vec2(ac0_15x0_9.s, ac0_37x0_9.t));
		blurredColor += texture2D(gcolor, texcoord.st + vec2(-ac0_37x0_9.s, ac0_15x0_9.t));
		blurredColor += texture2D(gcolor, texcoord.st + vec2(ac0_37x0_9.s, -ac0_15x0_9.t));
		blurredColor += texture2D(gcolor, texcoord.st + vec2(-ac0_15x0_9.s, -ac0_37x0_9.t));
		blurredColor += texture2D(gcolor, texcoord.st + vec2(-ac0_15x0_9.s, ac0_37x0_9.t));
		blurredColor += texture2D(gcolor, texcoord.st + vec2(ac0_37x0_9.s, ac0_15x0_9.t));
		blurredColor += texture2D(gcolor, texcoord.st + vec2(-ac0_37x0_9.s, -ac0_15x0_9.t));
		blurredColor += texture2D(gcolor, texcoord.st + vec2(ac0_15x0_9.s, -ac0_37x0_9.t));
#endif
#ifdef USE_HIGH_QUALITY_BLUR
		blurredColor *= 0.02439f;	// /= 41.0f;
#else
		blurredColor *= 0.0625f;	// /= 16.0f;
#endif
	} else {
		blurredColor += getSampleWithBoundsCheck(vec2(0.0f, ac0_4.t));
		blurredColor += getSampleWithBoundsCheck(vec2(ac0_4.s, 0.0f));
		blurredColor += getSampleWithBoundsCheck(vec2(0.0f, -ac0_4.t));
		blurredColor += getSampleWithBoundsCheck(vec2(-ac0_4.s, 0.0f));
#ifdef USE_HIGH_QUALITY_BLUR
		blurredColor += getSampleWithBoundsCheck(vec2(ac0_4x0_7.s, 0.0f));
		blurredColor += getSampleWithBoundsCheck(vec2(0.0f, -ac0_4x0_7.t));
		blurredColor += getSampleWithBoundsCheck(vec2(-ac0_4x0_7.s, 0.0f));
		blurredColor += getSampleWithBoundsCheck(vec2(0.0f, ac0_4x0_7.t));
		blurredColor += getSampleWithBoundsCheck(vec2(ac0_4x0_4.s, 0.0f));
		blurredColor += getSampleWithBoundsCheck(vec2(0.0f, -ac0_4x0_4.t));
		blurredColor += getSampleWithBoundsCheck(vec2(-ac0_4x0_4.s, 0.0f));
		blurredColor += getSampleWithBoundsCheck(vec2(0.0f, ac0_4x0_4.t));
#endif
		blurredColor += getSampleWithBoundsCheck(vec2(ac0_29.s, -ac0_29.t));
		blurredColor += getSampleWithBoundsCheck(vec2(ac0_29.s, ac0_29.t));
		blurredColor += getSampleWithBoundsCheck(vec2(-ac0_29.s, ac0_29.t));
		blurredColor += getSampleWithBoundsCheck(vec2(-ac0_29.s, -ac0_29.t));
#ifdef USE_HIGH_QUALITY_BLUR
		blurredColor += getSampleWithBoundsCheck(vec2(ac0_29x0_7.s, ac0_29x0_7.t));
		blurredColor += getSampleWithBoundsCheck(vec2(ac0_29x0_7.s, -ac0_29x0_7.t));
		blurredColor += getSampleWithBoundsCheck(vec2(-ac0_29x0_7.s, ac0_29x0_7.t));
		blurredColor += getSampleWithBoundsCheck(vec2(-ac0_29x0_7.s, -ac0_29x0_7.t));
		blurredColor += getSampleWithBoundsCheck(vec2(ac0_29x0_4.s, ac0_29x0_4.t));
		blurredColor += getSampleWithBoundsCheck(vec2(ac0_29x0_4.s, -ac0_29x0_4.t));
		blurredColor += getSampleWithBoundsCheck(vec2(-ac0_29x0_4.s, ac0_29x0_4.t));
		blurredColor += getSampleWithBoundsCheck(vec2(-ac0_29x0_4.s, -ac0_29x0_4.t));
#endif
		blurredColor += getSampleWithBoundsCheck(vec2(ac0_15.s, ac0_37.t));
		blurredColor += getSampleWithBoundsCheck(vec2(-ac0_37.s, ac0_15.t));
		blurredColor += getSampleWithBoundsCheck(vec2(ac0_37.s, -ac0_15.t));
		blurredColor += getSampleWithBoundsCheck(vec2(-ac0_15.s, -ac0_37.t));
		blurredColor += getSampleWithBoundsCheck(vec2(-ac0_15.s, ac0_37.t));
		blurredColor += getSampleWithBoundsCheck(vec2(ac0_37.s, ac0_15.t));
		blurredColor += getSampleWithBoundsCheck(vec2(-ac0_37.s, -ac0_15.t));
		blurredColor += getSampleWithBoundsCheck(vec2(ac0_15.s, -ac0_37.t));
#ifdef USE_HIGH_QUALITY_BLUR
		blurredColor += getSampleWithBoundsCheck(vec2(ac0_15x0_9.s, ac0_37x0_9.t));
		blurredColor += getSampleWithBoundsCheck(vec2(-ac0_37x0_9.s, ac0_15x0_9.t));
		blurredColor += getSampleWithBoundsCheck(vec2(ac0_37x0_9.s, -ac0_15x0_9.t));
		blurredColor += getSampleWithBoundsCheck(vec2(-ac0_15x0_9.s, -ac0_37x0_9.t));
		blurredColor += getSampleWithBoundsCheck(vec2(-ac0_15x0_9.s, ac0_37x0_9.t));
		blurredColor += getSampleWithBoundsCheck(vec2(ac0_37x0_9.s, ac0_15x0_9.t));
		blurredColor += getSampleWithBoundsCheck(vec2(-ac0_37x0_9.s, -ac0_15x0_9.t));
		blurredColor += getSampleWithBoundsCheck(vec2(ac0_15x0_9.s, -ac0_37x0_9.t));
#endif
		blurredColor /= samples;
	}
	return blurredColor;
}

vec4 getSampleWithBoundsCheck(in vec2 offset) {
	vec2 coord = texcoord.st + offset;
	if (coord.s <= 1.0f && coord.s >= 0.0f && coord.t <= 1.0f && coord.t >= 0.0f) {
		samples += 1.0f;
		return texture2D(gcolor, coord);
	}
    return vec4(0.0f);
}
#endif
