#version 120

uniform sampler2D texture;

uniform float frameTimeCounter;

varying vec4 color;
varying vec4 texcoord;
varying vec4 lmcoord;
varying vec4 position;

varying vec3 tangent;
varying vec3 binormal;
varying vec3 normal;
varying vec3 worldpos;

varying float iswater;
varying float icewater;

//////////////////////////////main//////////////////////////////

void main() {
    vec4 frag2 = vec4(normal * 0.5f + 0.5f, 1.0f);
    vec4 frag3 = frag2;
    vec4 tex = texture2D(texture, texcoord.xy);
    if (iswater > 0.9f) {
        // Mixing water texture with water color
        tex = mix(tex, vec4(0.02f, 0.03f, 0.14f, 0.85f), 0.5f);
        vec3 pos = worldpos;
        pos.x += sin(pos.z + frameTimeCounter) * 0.2f;
        pos.z += cos(pos.x + frameTimeCounter * 0.5f) * 0.2f;
        
        float wave2 = (sin(6.283f * (frameTimeCounter + pos.x  + pos.z * 0.5f)) + sin(6.283f * (frameTimeCounter * 1.2f + pos.x * 0.5f + pos.z))) * 0.1f;
        float bumpmult2 = 0.05f * pow(dot(normal, -normalize(position.xyz)), 0.5f);
        vec3 bump2 = vec3(sin(wave2 * 3.142f), 1.0f - cos(wave2 * 3.142f), wave2);
        bump2 = bump2 * bumpmult2 + vec3(0.0f, 0.0f, 1.0f - bumpmult2);
        mat3 tbnMatrix = mat3(tangent, binormal, normal);
        frag2 = vec4(tbnMatrix * bump2 * 0.5f + 0.5f, 1.0f);
        
        // Reflection heavily bump normal
        float wave3 = 0.6f * wave2;
        vec3 bump3 = vec3(1.0f - cos(wave3 * 3.142f), sin(wave3 * 3.142f), wave3);
        bump3 *= bumpmult2;
        frag3 = vec4(tbnMatrix * bump3 * 0.5f + 0.5f, 1.0f);
    }
    
/* DRAWBUFFERS:01234 */
    gl_FragData[0] = vec4(vec3(clamp(lmcoord.t + lmcoord.s, 0.25f, 1.0f)), 1.0f) * tex * color;
    gl_FragData[1] = frag3;
    gl_FragData[2] = frag2;
    gl_FragData[3] = gl_FragData[0];
    gl_FragData[4] = vec4(0.0f, icewater, lmcoord.s, 1.0f); // x = spec; y = basic, textured(0.0), shadow exit(0.1), lit(0.3), hand(0.4), entity(0.6), ice(0.9), water(1.0); z = torch lightmap
}

//////////////////////////////main//////////////////////////////
